use strict;

# configuration variables
my @desc; # strings at beginning of page
my @edesc; # strings at end of page (before ^L)

if ($ARGV[0] eq "--") {
	shift @ARGV;
} elsif ($ARGV[0] eq "-h") {
help:
	print STDERR <<EOT;
This is pclmodify 1.0 (c) 2003 by SANFACE Software.

Usage:
$0 control.txt pcl.prn >lptx:

The pclmodify program accompanies pdfmodify. You might want to read the docs
inside documentation dictionary or directly at the homepage of this tool

	http://www.sanface.com/pclmodify.html .

This program is NO FREEWARE.
EOT

exit 1;

} elsif (not defined $ARGV[1] || defined $ARGV[2]) {
	goto help;
}

# read config file
sub unquot {
	my $data = shift;
	my %trans = (
		"n" => "\n",
		"r" => "\r",
		"e" => "\x1b",
		"E" => "\x1b",
	);
	$data =~ s/\\(.)/
		defined $trans{$1} ? $trans{$1} : $1
	/ge;
	return $data;
}
my $l = 0;
open IN, "<$ARGV[0]" or die "could not open '$ARGV[0]'\n";
while (<IN>) {
	$l++;
	if (/^\s*(\d+)\s*:(.*)$/) {
		if (not defined $desc[$1-1]) { # at start of page
			$desc[$1-1] = "";
		}
		$desc[$1-1] .= unquot($2);
	} elsif (/^\s*e\s*(\d+)\s*:(.*)$/) { # at end of page
		if (not defined $edesc[$1-1]) {
			$edesc[$1-1] = "";
		}
		$edesc[$1-1] .= unquot($2);
	} elsif (not(/^\s*#/ or /^\s*$/)) {
		warn "$0: configuration file error at line $l\n";
	}
}
close IN;

# read pcl input
open IN, "<$ARGV[1]" or die "could not open '$ARGV[1]'\n";
binmode IN;
my $in = join('', <IN>);
close IN;

# core of that all: parse the page starts
sub pclpages($) {
    my $input = shift;
    pos($input) = my $p = 0;
    my $output = '';
    my $q;
    my $lpos = 0; # starting position for first page
    my @ppos; # positions of the page starts
    my @epos; # positions of the page ends
    while ($q < length($input)) {
	if ($input =~ /([\x1b\xc])/g) {
	  if ($1 eq "\xc") { # pagefeed
		push @ppos, $lpos;
		$lpos = pos($input);
		push @epos, $lpos - 1;
		next;
	  }
	  $q = $-[0];
	  if ($input =~ /\G[=9]/g) {
		# stupid things
	  } else { # usual escape
	    pos($input) = $q + 1;
	    if ($input =~ /\G([^\@A-Z]*)([\@A-Z])/g) {
		my $par = $1;
		my $cmd = $2;
		if ($cmd eq "E" && $par eq "") { # reset
			$lpos = pos($input);
		} elsif ($cmd eq "X") {
			if ($par =~ /^\&p(\d+)$/) {
				pos($input) += $1;
			}
		} elsif ($cmd eq "W") {
			if ($par =~ /^\&n(\d+)$/ ||
				$par =~ /^\([fs](\d+)$/ ||
				$par =~ /^\)s(\d+)$/ ||
				$par =~ /^\*b(\d+)$/) {

				pos($input) += $1;
			}
		} elsif ($cmd eq "V") {
			if ($par =~ /^\*b(^\d+)$/) {
				pos($input) += $1;
			}
		}
	    } else { # strange situation...
		pos($input) = $q + 1;
		goto again;
	    }
          }
	} else { # EOI
		$q = length($input);
	}
	$p = pos($input);
    }
    return { ppos => \@ppos, epos => \@epos };
}

# main program
binmode STDOUT;
my $pclpages = pclpages($in);
my $ppos = $pclpages->{ppos};
my $epos = $pclpages->{epos};
my $p = 0;
for (my $i = 0; $i <= $#$ppos; $i++) {
	print substr($in, $p, $$ppos[$i] - $p);
	print $desc[$i] if defined $desc[$i];
	$p = $$ppos[$i];
	print substr($in, $p, $$epos[$i] - $p);
	print $edesc[$i] if defined $edesc[$i];
	$p = $$epos[$i];
}
print substr($in, $p);

exit 0;
